import Foundation

/**
Stellt eine Scheibe des Turms auf dem Bildschirm dar

- author: Albert Wiedemann
- version: 1.0
*/
class ScheibenSymbol
{
    /** Basisgröße der Scheiben */
    static let basis = 20
    /** Der Außenrand */
    private let außen: Rechteck
    /** Der Innenrand */
    private let innen: Rechteck
    /** Breite der Scheibe */
    private let breite: Int
    /** x-Position der Scheibe */
    private var x: Int
    /** y-Position der Scheibe */
    private var y: Int

    /**
	Legt die beiden Rechtecke an und positioniert Sie auf Turm 1
	- parameters:
		- nummer die Scheibennummer
    */
    init(nummer: Int)
    {
        x = 0
        y = 0
        breite = ScheibenSymbol.basis * nummer
        außen = Rechteck()
        außen.FarbeSetzen(farbe: "schwarz")
        außen.GrößeSetzen(breite: ScheibenSymbol.basis * nummer, höhe: ScheibenSymbol.basis)
        innen = Rechteck()
        innen.FarbeSetzen(farbe: "rot")
        innen.GrößeSetzen(breite: ScheibenSymbol.basis * nummer - 2, höhe: ScheibenSymbol.basis - 2)
        PositionSetzen(turmNummer: 1, scheibenNummer: 1)
    }
    
    /**
	Setzt die Position bei der Lage in einem Turm.
	- parameters:
		- turmNummer: Nummer des Turms
		- scheibenNummer: Nummer der Scheibe von unten her ab 1.
     */
    func PositionSetzen (turmNummer: Int, scheibenNummer: Int)
    {
        let abstand = Darstellung.turmAbstand
        x = abstand / 2 + abstand * turmNummer - außen.breite / 2
        y = Darstellung.bodenPosition - ScheibenSymbol.basis * (scheibenNummer + 1)
        außen.PositionSetzen(x: x, y: y)
        innen.PositionSetzen(x: x + 1, y: y + 1)
    }
    
    /**
    Bewegt die Scheibe um zwei Punkte nach oben.
    */
    func Anheben ()
    {
        y -= 2
        außen.PositionSetzen(x: x, y: y)
        innen.PositionSetzen(x: x + 1, y: y + 1)
    }
    
    /**
    Bewegt die Scheibe um zwei Punkte nach unten.
    */
    func Absenken ()
    {
        y += 2
        außen.PositionSetzen(x: x, y: y)
        innen.PositionSetzen(x: x + 1, y: y + 1)
    }
    
    /**
    Bewegt die Scheibe um zwei Punkte nach links.
    */
    func LinksFahren ()
    {
        x -= 2
        außen.PositionSetzen(x: x, y: y)
        innen.PositionSetzen(x: x + 1, y: y + 1)
    }
    
    /**
    Bewegt die Scheibe um zwei Punkte nach rechts.
    */
    func RechtsFahren ()
    {
        x += 2
        außen.PositionSetzen(x: x, y: y)
        innen.PositionSetzen(x: x + 1, y: y + 1)
    }
}

/**
 Verwaltet den Kran

- author; Albert Wiedemann
- version: 1.0
 */
class Kran
{
    /** Der Querträger */
    private let träger: Rechteck
    /** Das Seil */
    private let seil: Rechteck
    /** Der Fanghaken */
    private let haken: Rechteck
     /** Das linke Rad */
    private let rad1: Kreis
     /** Das rechte Rad */
    private let rad2: Kreis
    /** x-Position des Krans */
    private var x: Int
    /** y-Position des Fanghakens */
    private var y: Int

    /**
    Legt die Figuren an und positioniert den Kran über auf Turm 1
    */
    init()
    {
        x = Darstellung.turmAbstand / 2 - 15
        y = 100
        träger = Rechteck()
        träger.FarbeSetzen(farbe: "blau")
        träger.GrößeSetzen(breite: 30, höhe: 6)
        seil = Rechteck()
        seil.FarbeSetzen(farbe: "blau")
        seil.GrößeSetzen(breite: 2, höhe: y - 60)
        haken = Rechteck()
        haken.FarbeSetzen(farbe: "blau")
        haken.GrößeSetzen(breite: 10, höhe: 2)
        rad1 = Kreis()
        rad1.FarbeSetzen(farbe: "blau")
        rad1.RadiusSetzen(radius: 5)
        rad2 = Kreis()
        rad2.FarbeSetzen(farbe: "blau")
        rad2.RadiusSetzen(radius: 5)
        PositionSetzen(turmNummer: 0)
    }
    
    /**
	Setzt die Position bei der Lage in einem Turm.
	- parameters:
		- turmNummer: Nummer des Turms
    */
    func PositionSetzen(turmNummer: Int)
    {
        x = Darstellung.turmAbstand / 2  + turmNummer * Darstellung.turmAbstand - 15
        träger.PositionSetzen(x: x, y: 57)
        seil.PositionSetzen(x: x + 14, y: 60)
        seil.GrößeSetzen(breite: 2, höhe: y - 60)
        haken.PositionSetzen(x: x + 10, y: y - 2)
        rad1.PositionSetzen(x: x, y: 60)
        rad2.PositionSetzen(x: x + 30, y: 60)
    }
    
    /**
	Bewegt die Scheibe um zwei Punkte nach oben.
    */
    func Anheben()
    {
        y -= 2
        träger.PositionSetzen(x: x, y: 57)
        seil.PositionSetzen(x: x + 14, y: 60)
        seil.GrößeSetzen(breite: 2, höhe: y - 60)
        haken.PositionSetzen(x: x + 10, y: y - 2)
        rad1.PositionSetzen(x: x, y: 60)
        rad2.PositionSetzen(x: x + 30, y: 60)
    }
    
    /**
	Bewegt die Scheibe um zwei Punkte nach unten.
    */
    func Absenken()
    {
        y += 2
        träger.PositionSetzen(x: x, y: 57)
        seil.PositionSetzen(x: x + 14, y: 60)
        seil.GrößeSetzen(breite: 2, höhe: y - 60)
        haken.PositionSetzen(x: x + 10, y: y - 2)
        rad1.PositionSetzen(x: x, y: 60)
        rad2.PositionSetzen(x: x + 30, y: 60)
    }
    
    /**
	Bewegt die Scheibe um zwei Punkte nach links.
    */
    func LinksFahren()
    {
        x -= 2
        träger.PositionSetzen(x: x, y: 57)
        seil.PositionSetzen(x: x + 14, y: 60)
        haken.PositionSetzen(x: x + 10, y: y - 2)
        rad1.PositionSetzen(x: x, y: 60)
        rad2.PositionSetzen(x: x + 30, y: 60)
    }
    
    /**
	Bewegt die Scheibe um zwei Punkte nach rechts.
    */
    func RechtsFahren()
    {
        x += 2
        träger.PositionSetzen(x: x, y: 57)
        seil.PositionSetzen(x: x + 14, y: 60)
        haken.PositionSetzen(x: x + 10, y: y - 2)
        rad1.PositionSetzen(x: x, y: 60)
        rad2.PositionSetzen(x: x + 30, y: 60)
    }
    
    /**
	Gibt die x-Position des Krans bezogen auf die Seilmitte
	- returns: Kranposition
    */
    func KranpositionGeben() -> Int
    {
        return x + 15
    }
    
    /**
	Gibt die y-Position der Unterkante des Kranhakens
	- returns: Kranhöhe
    */
    func KranhöheGeben() -> Int
    {
        return y
    }
}

/**
Verwaltet die Vísualisierung

- author: Albert Wiedemann
- version: 1.0
*/
class Darstellung: Ereignisbehandlung
{
    /** Abstand der Türme */
    static let turmAbstand = 250
    /** Position des Bodens */
    static let bodenPosition = 400
    /** Boden */
    private let boden: Rechteck
    /** Laufschiene */
    private let schiene: Rechteck
    /** Erster Turm */
    private let turm1: Rechteck
    /** Zweiter Turm */
    private let turm2: Rechteck
    /** Dritter Turm */
    private  let turm3: Rechteck
    /** Der Laufkran */
    private let kran: Kran
    /** Die Scheibenstapel */
    private var türme: [[ScheibenSymbol]]
    /** Tickzähler für die Wartezeit */
    private var wartezeitZähler: Int

    /**
	Baut die Darstellung auf und verwaltet die Türme und den Kran
	- parameters:
		- scheibenAnzahl: die Anzahl der Scheiben, die zu Beginn auf dem ersten Turm liegt
    */
    init(scheibenAnzahl: Int)
    {
        wartezeitZähler = 0
        boden = Rechteck()
        boden.FarbeSetzen(farbe: "schwarz")
        boden.PositionSetzen(x: 0, y: Darstellung.bodenPosition)
        boden.GrößeSetzen(breite: Darstellung.turmAbstand * 3, höhe: 5)
        schiene = Rechteck()
        schiene.FarbeSetzen(farbe: "schwarz")
        schiene.PositionSetzen(x: 0, y: 50)
        schiene.GrößeSetzen(breite: Darstellung.turmAbstand * 3, höhe: 5)
        turm1 = Rechteck()
        turm1.FarbeSetzen(farbe: "schwarz")
        turm1.PositionSetzen(x: Darstellung.turmAbstand / 2 - 3, y: Darstellung.bodenPosition - 200)
        turm1.GrößeSetzen(breite: 6, höhe: 200)
        turm2 = Rechteck()
        turm2.FarbeSetzen(farbe: "schwarz");
        turm2.PositionSetzen(x: Darstellung.turmAbstand * 3 / 2 - 3, y: Darstellung.bodenPosition - 200)
        turm2.GrößeSetzen(breite: 6, höhe: 200)
        turm3 = Rechteck()
        turm3.FarbeSetzen(farbe: "schwarz")
        turm3.PositionSetzen(x: Darstellung.turmAbstand * 5 / 2 - 3, y: Darstellung.bodenPosition - 200)
        turm3.GrößeSetzen(breite: 6, höhe: 200)
        kran = Kran()
        türme = [[ScheibenSymbol]]()
        türme.append([ScheibenSymbol]())
        türme.append([ScheibenSymbol]())
        türme.append([ScheibenSymbol]())
        for i in 0 ..< scheibenAnzahl
        {
            let neu = ScheibenSymbol(nummer: scheibenAnzahl - i)
            neu.PositionSetzen(turmNummer: 0, scheibenNummer: i)
            türme[0].append(neu)
        }
        super.init()
        TaktdauerSetzen(dauer: 100)
    }
    
    /**
	Die eigentliche Aktionsmethode des Zeitgebers.
	Erniedrigt den Zähler für die Wartezeit
    */
    override func TaktImpulsAusführen()
    {
        wartezeitZähler -= 1
    }
    
    /**
	Bremst den Ablauf, um eine beobachtbare Bewegung zu erhalten.
	- parameters:
		- ticks: die Anzahl der abzuwartenden Ticks der Ereignisbehandlung
    */
    private func Warten(ticks: Int)
    {
        wartezeitZähler = ticks
        
        repeat
        {
            Thread.sleep(forTimeInterval: 10)
        }
        while (wartezeitZähler > 0)
    }
    
    /**
	Senkt den Kran, gegebenenfalls mit Scheibe, auf die gewünschte Höhe ab.
	- parameters:
		- zielHöhe: die gewünschte Höhe
		- scheibe: Referenz der bewegten Scheibe.
    */
    private func Absenken(zielHöhe: Int, scheibe: ScheibenSymbol?)
    {
        while zielHöhe > kran.KranhöheGeben()
        {
            kran.Absenken ()
            if scheibe != nil
            {
                scheibe!.Absenken()
                Warten(ticks: 3)
            }
            else
            {
                Warten(ticks: 1)
            }
        }
    }
    
    /**
	Hebt den Kran, gegebenenfalls mit Scheibe, auf die Fahrhöhe an.
	- parameters:
		- scheibe: Referenz der bewegten Scheibe.
     */
    private func Anheben(scheibe: ScheibenSymbol?)
    {
        while 100 < kran.KranhöheGeben()
        {
            kran.Anheben()
            if scheibe != nil
            {
                scheibe!.Anheben()
                Warten (ticks: 3)
            }
            else
            {
                Warten (ticks: 1)
            }
        }
    }
    
    /**
	Fährt den Kran, gegebenenfalls mit Scheibe, auf die gewünschte Position nach links.
	- parameters:
		- zielPosition: die gewünschte Position
		- scheibe: Referenz der bewegten Scheibe.
     */
    private func LinksFahren(zielPosition: Int, scheibe: ScheibenSymbol?)
    {
        while zielPosition < kran.KranpositionGeben()
        {
            kran.LinksFahren()
            if scheibe != nil
            {
                scheibe!.LinksFahren()
                Warten (ticks: 3)
            }
            else
            {
                Warten (ticks: 1)
            }
        }
    }
    
    /**
	Fährt den Kran, gegebenenfalls mit Scheibe, auf die gewünschte Position nach rechts.
	- parameters:
		- zielPosition: die gewünschte Position
		- scheibe: Referenz der bewegten Scheibe.
     */
    private func RechtsFahren(zielPosition: Int, scheibe: ScheibenSymbol?)
    {
        while zielPosition > kran.KranpositionGeben()
        {
            kran.RechtsFahren()
            if scheibe != nil
            {
                scheibe!.RechtsFahren()
                Warten (ticks: 3)
            }
            else
            {
                Warten (ticks: 1)
            }
        }
    }

    /**
	Bewegt die oberste Scheibe vom Startturm zum Zielturm.
	- parameters:
		- start: Nummer des Startturms
		- ziel: Nummer des Zielturms
    */
    func Bewegen(start: Int, ziel: Int)
    {
        var scheibenanzahl = türme[start].count
        let scheibe = türme[start].remove(at: scheibenanzahl - 1)
        var position = Darstellung.turmAbstand / 2 + Darstellung.turmAbstand * start
        if position < kran.KranpositionGeben()
        {
            LinksFahren(zielPosition: position, scheibe: nil)
        }
        else if position > kran.KranpositionGeben()
        {
            RechtsFahren (zielPosition: position, scheibe: nil)
        }
        Absenken(zielHöhe: Darstellung.bodenPosition - ScheibenSymbol.basis * scheibenanzahl, scheibe: nil)
        Anheben(scheibe: scheibe)
        position = Darstellung.turmAbstand / 2 + Darstellung.turmAbstand * ziel
        if position < kran.KranpositionGeben()
        {
            LinksFahren(zielPosition: position, scheibe: scheibe)
        }
        else if position > kran.KranpositionGeben()
        {
            RechtsFahren(zielPosition: position, scheibe: scheibe)
        }
        scheibenanzahl = türme[ziel].count + 1
        Absenken(zielHöhe: Darstellung.bodenPosition - ScheibenSymbol.basis * scheibenanzahl, scheibe: scheibe)
        türme[ziel].append(scheibe)
        Anheben(scheibe: nil)
    }
    
    /**
	Verschiebt die oberste Scheibe ohne Animation vom Startturm zum Zielturm.
	- parameters:
		- start: Nummer des Startturms
		- ziel: Nummer des Zielturms
    */
    func Verschieben (start: Int, ziel: Int)
    {
        let scheibe = türme[start].remove(at: türme[start].count - 1)
        türme[ziel].append(scheibe)
        scheibe.PositionSetzen(turmNummer: ziel, scheibenNummer: türme[ziel].count - 1)
        Warten(ticks: 1)
    }
}

/**
Steuert die "Türme von Hanoi"

- author: Albert Wiedemann
- version: 1.0
*/
class Hanoi
{
    /** Das Darstellungsobjekt */
    private let darstellung: Darstellung
    
    /**
	Legt die Darstellung an und startet den Ablauf
	- parameters:
		- anzahl: Anzahl der Scheiben auf dem Ausgangsturm
    */
    init(anzahl: Int)
    {
        darstellung = Darstellung(scheibenAnzahl: anzahl)
        darstellung.Starten()
        TurmVerschieben(anzahlScheiben: anzahl, von: 0, nach: 2)
    }
       
    /**
	Rekursive Steuermethode zur Bewegung des zu bewegenden (Teil-)Turms.
	- parameters:
		- anzahlScheiben: Anzahl der Scheiben des Turms
		- von Nummer: des Ausgangsturms
		- nach Nummer: des Zielturms
    */
    func TurmVerschieben(anzahlScheiben: Int, von: Int, nach: Int)
    {
        if anzahlScheiben > 0
        {
            //Rumpf der Methode ergänzen
        }
    }
    
    /**
	Bewegt die oberste Platte des Turms mit der Nummer von auf den Turm mit der Nummer nach
	- parameters:
		- von: Nummer des Ausgangsturms
		- nach: Nummer des Zielturms
     */
    private func Bewegen(von: Int, nach: Int)
    {
        darstellung.Bewegen(start: von, ziel: nach)
        //darstellung.Verschieben(start: von, ziel: nach)
    }
}

let türme = Hanoi(anzahl: 4)
türme.TurmBewegen(anzahlScheiben: 4, von: 0, nach: 1)
