/**
Beschreibung der Klasse Patient

- author: Klaus Reinold
- version: 1.0
*/
class Patient
{
    /** Name des Patienten */
    private var name: String

    /**
    Konstruktor des Patienten
    - parameters:
        - s: Name des Patienten
     */
    init(s: String)
    {
        name = s
    }

    /**
    Gibt Information über den Patienten aus.
    */
    func InformationAusgeben()
    {
        print(name)
    }

    /**
    Gibt den Namen des Patienten aus
    - returns: Name des Patienten
    */
    func NameGeben() -> String
    {
        return name
    }
    
    /**
    Gibt zurück, ob der Name gleich den übergebenen Wert ist.
    - parameters:
        - gesuchterWert: gesuchter Wert
    - returns: Wahrheitswert, der angibt, ob der Name der gesuchte ist.
    */
    func SchlüsselIstGleich(gesuchterWert: String) -> Bool
    {
        return name == gesuchterWert
    }
    
    /**
    Vergleich zweier Datenelemente bezüglich einer Ordnungsrelation
     - parameters:
         - dvergleich: Datenelement zum Vergleich
     - returns: Wahrheitswert, der angibt, ob der Name kleiner als der Vergleichswert ist
     */
    func IstKleinerAls(dvergleich: Patient) -> Bool
    {
        return name < dvergleich.name
    }
}

/**
 Abstrakte Klasse Listenelement als Oberklasse für Knoten und Abschluss

- author: Klaus Reinold
- version: 1.0
*/
class Listenelement
{

    /**
    Bestimmt die Länge der restlichen Liste ab dem aktuellen Knoten
    - returns: Länge der restlichen Liste
     ´*/
    func RestlängeGeben() -> Int
    {
        fatalError("Listenelement.RestlängeGeben muss überschrieben werden")
    }
    
    /**
     Gibt das Datenelement des Vorgängers (also des letzten Knotens) zurück
    - parameters:
        - dNeu übergebenes Datenelement
    - returns: hinterstes Datenelement
    */
    func EndeGeben(dNeu: Patient?) -> Patient?
    {
        fatalError("Listenelement.EndeGeben muss überschrieben werden")
    }

    /**
    Entfernt einen Knoten, der über sein Datenelement identifiziert wird, aus der Liste
    - parameters:
        - dvergleich: gesuchtes Datenelement
    - returns: Weitergabe des künftigen Nachfolgers
    */
    func KnotenEntfernen(dvergleich: Patient?) -> Listenelement
    {
        fatalError("Listenelement.KnotenEntfernen muss überschrieben werden")
    }
    
    /**
    Gibt das Datenelement eines Knotens zurück
    - returns: Datenelement
    */
    func DatenelementGeben() -> Patient?
    {
        fatalError("Listenelement.DatenelementGeben muss überschrieben werden")
    }

    /**
    Gibt den Nachfolger des Knoten aus.
    - returns: Nachfolger
    */
    func NachfolgerGeben() -> Listenelement
    {
        fatalError("Listenelement.NachfolgerGeben muss überschrieben werden")
    }
    
    /**
     Gibt die Information zu den Datenelementen auf der Konsole aus
    */
    func InformationAusgeben()
    {
        fatalError("Listenelement.InformationAusgeben muss überschrieben werden")
    }

    /**
    Fügt ein Datenelement am Ende der Liste ein
    - parameters:
        - dNeu: neues Datenelement
    */
    func HintenEinfügen(dNeu: Patient) -> KnotenListe
    {
        fatalError("Listenelement.HintenEinfügen muss überschrieben werden")
    }

    /**
    Fügt ein Datenelement vor einem bestimmten anderen Datenelement ein
    - parameters:
        - dNeu: neues Datenelement
        - dVergleich: gesuchtes Datenelement, vor dem eingefügt werden soll
    - returns: Weitergabe des künftigen Nachfolgers
    */
    func EinfügenVor(dNeu: Patient, dVergleich: Patient) -> KnotenListe
    {
        fatalError("Listenelement.EinfügenVor muss überschrieben werden")
    }
    
    /**
    Sucht ein Datenelement mit einem bestimmten Schlüsselwert in der Liste
    - parameters:
        - schlüssel: Zeichenkette, die als Schlüssel gefunden werden soll.
    - returns: das gesuchte Datenelement
    */
    func Suchen(schlüssel: String) -> Patient?
    {
        fatalError("Listenelement.Suchen muss überschrieben werden")
    }

    /**
     Sortiertes Einfügen in die Liste
    - parameters:
        - dNeu: neues Datenelement
    - returns: Knoten als neuer Nachfolger für den Vorgänger
    */
    func SortiertEinfügen(dNeu: Patient) -> KnotenListe
    {
        fatalError("Listenelement.SortiertEinfügen muss überschrieben werden")
    }
}

/**
 Klasse zur Kennzeichnung des Endes der Liste

- author: Klaus Reinold
- version: 1.0
*/
class AbschlussListe: Listenelement
{

    /**
    Bestimmt die Länge der restlichen Liste ab dem aktuellen Knoten
    - returns: Länge der restlichen Liste
    */
    override func RestlängeGeben() -> Int
    {
        return 0
    }
    
    /**
    Gibt das Datenelement des letzten Knotens aus
     - parameters:
         - dNeu übergebenes Datenelement
    - returns: übergebenes Datenelement
    */
    override func EndeGeben(dNeu: Patient?) -> Patient?
    {
        return dNeu
    }

    /**
    Entfernt einen Knoten, der über sein Datenelement identifiziert wird, aus der Liste
    - parameters:
        - dvergleich: gesuchtes Datenelement
    - returns: Weitergabe des künftigen Nachfolgers
    */
    override func KnotenEntfernen(dvergleich: Patient?) -> Listenelement
    {
        return self
    }
    
    /**
    Gibt das Datenelement eines Knotens zurück
    - returns: Datenelement
    */
    override func DatenelementGeben() -> Patient?
    {
        return nil
    }

    /**
     Da der Abschluss keinen Nachfolger hat, gibt er sich selbst zurück
    - returns: Nachfolger
    */
    override func NachfolgerGeben() -> Listenelement
    {
        return self
    }
    
    /**
     Gibt die Information zu den Datenelementen auf der Konsole aus
    */
    override func InformationAusgeben()
    {
    }

    /**
    Fügt ein Datenelement am Ende der Liste ein
    - parameters:
        - dNeu: neues Datenelement
    */
    override func HintenEinfügen(dNeu: Patient) -> KnotenListe
    {
        return KnotenListe(dNeu: dNeu, lNeu: self)
    }

    /**
    Fügt ein Datenelement vor einem bestimmten anderen Datenelement ein
    - parameters:
        - dNeu: neues Datenelement
        - dVergleich: gesuchtes Datenelement, vor dem eingefügt werden soll
    - returns: Weitergabe des künftigen Nachfolgers
    */
    override func EinfügenVor(dNeu: Patient, dVergleich: Patient) -> KnotenListe
    {
        return KnotenListe(dNeu: dNeu, lNeu: self)
    }
    
    /**
    Sucht ein Datenelement mit einem bestimmten Schlüsselwert in der Liste
    - parameters:
        - schlüssel: Zeichenkette, die als Schlüssel gefunden werden soll.
    - returns: das gesuchte Datenelement
    */
    override func Suchen(schlüssel: String) -> Patient?
    {
        return nil
    }

    /**
     Sortiertes Einfügen in die Liste
    - parameters:
        - dNeu: neues Datenelement
    - returns: Knoten als neuer Nachfolger für den Vorgänger
    */
    override func SortiertEinfügen(dNeu: Patient) -> KnotenListe
    {
        return KnotenListe(dNeu: dNeu, lNeu: self)
    }
}

/**
Beschreibung der Klasse Knoten

- author: Klaus Reinold
- version: 1.0
*/
class KnotenListe: Listenelement
{
    /** Verwaltung des nachfolgenden Knoten in der Liste. */
    private var nachfolger: Listenelement
    /** Verwaltung der Daten des Patienten */
    private var daten: Patient

    /**
    Konstruktor des Knoten
    - parameters:
        - dNeu: Datenelement
        - lNeu: neuer Nachfolger
    */
    init(dNeu: Patient, lNeu: Listenelement)
    {
        nachfolger  = lNeu
        daten = dNeu
    }

    /**
    Bestimmt die Länge der restlichen Liste ab dem aktuellen Knoten
    - returns: Länge der restlichen Liste
     ´*/
    override func RestlängeGeben() -> Int
    {
        return nachfolger.RestlängeGeben() + 1
    }
    
    /**
    Gibt das Datenelement des letzten Knotens aus
    - parameters:
        - dNeu: übergebenes Datenelement
    - returns: hinterstes Datenelement
    */
    override func EndeGeben(dNeu: Patient?) -> Patient?
    {
        return nachfolger.EndeGeben(dNeu: daten)
    }

    /**
     Entfernt den Knoten, dessen Datenelement das übergebene ist.
    - parameters:
        - dvergleich: gesuchtes Datenelement
    - returns: Weitergabe des künftigen Nachfolgers
    */
    override func KnotenEntfernen(dvergleich: Patient?) -> Listenelement
    {
        if daten === dvergleich
        {
            return nachfolger
        }
        else
        {
            nachfolger=nachfolger.KnotenEntfernen(dvergleich: dvergleich)
            return self
        }
    }

    /**
    Gibt den Nachfolger des Knoten aus.
    - returns: Nachfolger
    */
    override func NachfolgerGeben() -> Listenelement
    {
        return nachfolger
    }
    
    /**
    Gibt das Datenelement zurück
    - returns: Datenelement
    */
    override func DatenelementGeben() -> Patient
    {
        return daten
    }
    
    /**
     Gibt die Information zu den Datenelementen auf der Konsole aus
    */
    override func InformationAusgeben()
    {
        daten.InformationAusgeben()
        nachfolger.InformationAusgeben()
    }

    /**
    Fügt ein Datenelement am Ende der Liste ein
    - parameters:
        - dNeu: neues Datenelement
    */
    override func HintenEinfügen(dNeu: Patient) -> KnotenListe
    {
        nachfolger = nachfolger.HintenEinfügen(dNeu: dNeu)
        return self
    }

    /**
    Fügt ein Datenelement vor einem bestimmten anderen Datenelement ein
    - parameters:
        - dNeu: neues Datenelement
        - dVergleich: gesuchtes Datenelement, vor dem eingefügt werden soll
    - returns: Weitergabe des künftigen Nachfolgers
    */
    override func EinfügenVor(dNeu: Patient, dVergleich: Patient) -> KnotenListe
    {
        if daten === dVergleich
        {
            return KnotenListe(dNeu: dNeu, lNeu: self)
        }
        else
        {
            nachfolger = nachfolger.EinfügenVor(dNeu: dNeu, dVergleich: dVergleich)
            return self
        }
    }
    
    /**
    Sucht ein Datenelement mit einem bestimmten Schlüssel in der Liste
    - parameters:
        - schlüssel: Zeichenkette, die als Schlüssel gefunden werden soll.
    - returns: das gesuchte Datenelement
    */
    override func Suchen(schlüssel: String) -> Patient?
    {
        if daten.SchlüsselIstGleich(gesuchterWert: schlüssel)
        {
            return daten
        }
        else
        {
            return nachfolger.Suchen(schlüssel: schlüssel)
        }
    }

    /**
    Fügt ein Datenelement sortiert in die Liste ein
    - parameters:
        - dNeu: neues Datenelement
    - returns: Weitergabe des künftigen Nachfolgers
    */
    override func SortiertEinfügen(dNeu: Patient) -> KnotenListe
    {
        if daten.IstKleinerAls(dvergleich: dNeu)
        {
            nachfolger = nachfolger.SortiertEinfügen(dNeu: dNeu)
            return self
        }
        else
        {
            return KnotenListe(dNeu: dNeu, lNeu: self)
        }
    }
}

/**
Beschreibung der Liste

- author: Klaus Reinold
- version: 1.0
*/
class Liste
{
    /** erstes Element der Liste */
    private var anfang: Listenelement

    /**
    Konstruktor der Liste
    */
    init()
    {
        anfang = AbschlussListe()
    }

    /**
    Fügt ein neues Datenelement am Anfang der Liste ein.
    - parameters:
        - dNeu: neues Datenelement
    */
    func VorneEinfügen(dNeu: Patient)
    {
        anfang = KnotenListe(dNeu: dNeu, lNeu: anfang)
    }

    /**
    Bestimmt die Länge der Liste
    - returns: Listenlänge
    */
    func LängeGeben() -> Int
    {
        return anfang.RestlängeGeben()
    }
    
    /**
    Entfernt den letzten Knoten und gibt dessen Datenelement zurück.
    - returns: Datenelement des bisher letzten Knotens
    */
    func EndeEntfernen() -> Patient?
    {
        let dletzter = anfang.EndeGeben(dNeu: nil)
        anfang = anfang.KnotenEntfernen(dvergleich: dletzter)
        return dletzter
    }

    /**
    Entfernt den ersten Knoten aus der Liste und gibt sein Datenelement aus.
    - returns: Datenelement des bisher ersten Knotens
    */
    func AnfangEntfernen() -> Patient?
    {
        let d = anfang.DatenelementGeben()
        anfang = anfang.NachfolgerGeben()
        return d
    }

    /**
    Gibt Information über die Liste aus.
    */
    func InformationAusgeben()
    {
        anfang.InformationAusgeben()
    }

    /**
     Fügt einen neuen Patienten am Ende der Liste ein.
    - parameters:
        - dNeu: neues Datenelement
    */
    func HintenEinfügen(dNeu: Patient)
    {
        anfang = anfang.HintenEinfügen(dNeu: dNeu)
    }

    /**
    Fügt ein neues Datenelement vor einem bestimmten Datenelement ein.
    - parameters:
        - dNeu: neues Datenelement
        - dVergleich: Datenelement, vor dem eingefügt werden soll
    */
    func EinfügenVor(dNeu: Patient, dVergleich: Patient)
    {
        anfang = anfang.EinfügenVor(dNeu: dNeu, dVergleich: dVergleich)
    }

    /**
    Sucht ein Datenelement mit einem bestimmten Schlüssel in der Liste
    - parameters:
        - schlüssel: Zeichenkette, die als Schlüssel gefunden werden soll.
    - returns: das gesuchte Datenelement
    */
    func Suchen(schlüssel: String) -> Patient?
    {
        return anfang.Suchen(schlüssel: schlüssel)
    }

    /**
    Fügt ein neues Datenelement sortiert in die Liste ein.
    - parameters:
        - dNeu: neues Datenelement
    */
    func SortiertEinfügen(dNeu: Patient)
    {
        anfang = anfang.SortiertEinfügen(dNeu: dNeu)
    }

    /**
    Entfernt einen Knoten mit einem bestimmten Datenelement aus der Liste
    - parameters:
        - dvergleich: gesuchtes Datenelement
    */
    func KnotenEntfernen(dvergleich: Patient)
    {
        anfang = anfang.KnotenEntfernen(dvergleich: dvergleich)
    }
}



/**
Die Klasse Wörterbucheintrag dient zur Speicherung von
Wörterbucheinträgen für ein Englisch-Deutsches
Wörterbuch. Dabei kann jedem englischen Wort mehrere
deutsche Bedeutungen zugeordnet werden, wobei alle Bedeutungen in
einer einzigen Zeichenkette Bedeutung gespeichert werden.

- author: Peter Brichzin
- version: 23.05.24
*/
class Wörterbucheintrag
{
    /** Englisches Wort; Schlüssel dieses Datenelements */
    private var wort: String
    /** Deutsche Bedeutung; mehrere Bedeutungen werden als eine Zeichenkette zusammengefasst*/
    private var bedeutung: String

    /**
    Konstruktor für Objekte der Klasse Woerterbucheintrag
    - parameters:
        - wortNeu: das englische Wort.
        - bedeutungNeu: die deutsche(n) Bedeutung(en).
    */
    init(wortNeu: String, bedeutungNeu: String)
    {
        wort = wortNeu
        bedeutung = bedeutungNeu
    }

    /**
    Gibt Information über das Datenelement zu Kontrollzwecken
    auf das Terminalfenster aus.
    */
    func InformationAusgeben()
    {
        print(wort + ": " + bedeutung)
    }

    /**
    Vergleicht zwei Datenelemente auf Gleichheit.
    - parameters::
        - dvergleich: Datenelement mit dem das Objekt verglichen wird.
    - returns: true, wenn die beiden Datenelemente gleichen Schlüssel haben.
    */
    func IstGleich(dvergleich: Wörterbucheintrag) -> Bool
    {
        return wort == dvergleich.WortGeben()
    }

    /**
    Vergleicht zwei Datenelemente bezüglich der Ordnungsrelation.
    - parameters:
        - vergleichselement: Datenelement mit dem das Objekt verglichen wird.
    - returns: true, wenn das aktuelle Element einen größeren Schlüssel hat, als das angegebene Vergleichselement.
    */
    func IstGrößerAls(dvergleich: Wörterbucheintrag) -> Bool
    {
        return wort > dvergleich.WortGeben()
    }

    /**
    Vergleicht zwei Schlüssel, die als Zeichenketten vorliegen, auf Gleichheit.
    - parameters:
        - vergleichsSchlüssel Schlüssel  mit dem der Schlüssel des Objekt verglichen wird.
    - returns: true, wenn die beiden Schlüssel gleich sind.
    */
    func SchlüsselIstGleich(vergleichsSchlüssel: String) -> Bool
    {
        return wort == vergleichsSchlüssel
    }

    /**
    Vergleicht zwei Schlüssel, die als Zeichenketten vorliegen, bezüglich der Ordnungsrelation.
    - parameters:
        - vergleichsSchlüssel: Schlüssel  mit dem der Schlüssel des Objekt verglichen wird.
     * @return true, wenn das aktuelle Element einen größeren Schlüssel hat, als das angegebene Vergleichselement.
     */
    func SchlüsselIstGrößerAls(vergleichsSchlüssel: String) -> Bool
    {
        return wort > vergleichsSchlüssel
    }

    /**
    Vergleicht zwei Schlüssel, die als Zeichenketten vorliegen, bezüglich der Ordnungsrelation.
    - parameters:
        - vergleichsSchlüssel: Schlüssel  mit dem der Schlüssel des Objekt verglichen wird.
     * @return true, wenn das aktuelle Element einen kleineren Schlüssel hat, als das angegebene Vergleichselement.
     */
    func SchlüsselIstKleinerAls(vergleichsSchlüssel: String) -> Bool
    {
        return wort < vergleichsSchlüssel
    }

    /**
    Gibt den Schlüssel des Datenelements als String aus, auch wenn der Datentyp vom
    String abweicht. Diese Methode wird beim Entfernen eines Knotens benötigt.
    - returns: Schlüssel als String.
    */
    func SchlüsselAlsStringGeben() -> String
    {
        return wort
    }

    /**
    Geben-Methode zum Attribut wort
    - returns: das englische Wort
    */
    func WortGeben() -> String
    {
        return wort
    }

    /**
    Geben-Methode zum Attribut bedeutung
    - returns: bedeutung: die deutsche(n) Bedeutung(en) als eine Zeichenkette
    */
    func BedeutungGeben() -> String
    {
        return bedeutung
    }

    /**
    Setzen-Methode zum Attribut bedeutung
    - parameters:
        - bedeutungNeu: neue Bedeutung für das englische Wort
    */
    func BedeutungSetzen(bedeutungNeu: String)
    {
        bedeutung = bedeutungNeu
    }
}

/**
Abstrakte Klasse Baumelement zur Realisierung einer Baumstruktur mit Hilfe des Entwurfsmusters
Kompositum: Die Klasse legt ausschließlich die Schnittstelle für die Klassen
Knoten und Abschluss in Form von Methoden fest.

- author: Peter Brichzin
- version: 23.5.24
*/
class Baumelement
{
    /**
    Der Konstruktor ist nur der Vollständigkeit halber angegeben.
    */
    init()
    {
        // nichts zu tun
    }

    /**
    Fügt ein Datenelement, falls sein Schlüssel noch nicht vorhanden ist,
    in die bestehende Datenstruktur ein. D.h. die Eigenschaft geordneter Binärbaum
    bleibt erhalten.
    - parameters:
        - datenNeu: das Datenelement des einzufügenden Knoten
    - returns: (neues) Nachfolgeelement des Aufrufers, null falls es kein Datenelement mit dem eingegebenen Schlüssel gibt
    */
    func Einfügen(datenNeu: Wörterbucheintrag) -> Baumelement
    {
        fatalError("Baumelement.Einfügen muss überschrieben werden")
    }

    /**
    Sucht ein Datenelement, das über seinen Schlüssel identifiziert wird
    und gibt dieses aus.
    - parameters:
        - suchSchlüssel: Schlüssel nach dem gesucht wird als Zeichenkette
    - returns: gesuchtes Datenelement bzw. null, falls die Suche erfolglos ist
    */
    func Suchen(suchSchlüssel: String) -> Wörterbucheintrag?
    {
        fatalError("Baumelement.Suchen muss überschrieben werden")
    }

    /**
    Überprüft, ob ein Datenelement mit dem eingegebenen Schlüssel (in Form einer
    Zeichenkette) vorhanden ist.
    - parameters:
        - suchSchlüssel: Schlüssel, nach dem gesucht wird als Zeichenkette
    - returns: true, im Erfolgsfall, false sonst.
    */
    func IstVorhanden(suchSchlüssel: String) -> Bool
    {
        fatalError("Baumelement.IstVorhanden muss überschrieben werden")
    }

    /**
    Berechnet die Höhe des Baums und gibt diesen Wert zurück
    - parameters:
        - aktTiefe: Tiefe des aktuellen Knotens
    - returns: die Höhe = maximale Tiefe (der bisher besuchten Knoten)
    */
    func HöheGeben(aktTiefe: Int) -> Int
    {
        fatalError("Baumelement.HöheGeben muss überschrieben werden")
    }

    /**
    Gibt die Tiefe des Knotens an, in dem ein Datenelement mit dem Schlüssel suchSchluessel gespeichert ist.
    - parameters:
        - suchSchlüssel: Schlüssel, nach dem gesucht wird als Zeichenkette
        - tiefe: die Tiefe des aktuellen Knotens
    - returns: Tiefe als positive Zahl, bzw. -1 falls der suchSchlüssel nicht vorhanden ist.
    */
    func TiefeGeben(suchSchlüssel: String, tiefe: Int) -> Int
    {
        fatalError("Baumelement.TiefeGeben muss überschrieben werden")
    }
}

/**
Klasse Abschluss als Bestandteil eines geordneten Binärbaums.
(Implementierung mit dem Entwurfsmuster Kompositum)

- author: Peter Brichzin
- version: 23.5.24
*/
class AbschlussBaum: Baumelement
{
    /**
    Der Konstruktor ist nur der Vollständigkeit halber angegeben.
    */
    override init()
    {
        super.init()
    }

    /**
    Fügt ein Datenelement, falls sein Schlüssel noch nicht vorhanden ist,
    in die bestehende Datenstruktur ein. D.h. die Eigenschaft geordneter Binärbaum
    bleibt erhalten.
    - parameters:
        - datenNeu: das Datenelement des einzufügenden Knoten
    - returns: (neues) Nachfolgeelement des Aufrufers, null falls es kein Datenelement mit dem eingegebenen Schlüssel gibt
    */
    override func Einfügen(datenNeu: Wörterbucheintrag) -> Baumelement
    {
        return KnotenBaum(datenNeu: datenNeu)
    }

    /**
    Sucht ein Datenelement, das über seinen Schlüssel identifiziert wird
    und gibt dieses aus.
    - parameters:
        - suchSchlüssel: Schlüssel nach dem gesucht wird als Zeichenkette
    - returns: gesuchtes Datenelement bzw. null, falls die Suche erfolglos ist
    */
    override func Suchen(suchSchlüssel: String) -> Wörterbucheintrag?
    {
        return nil
    }

    /**
    Überprüft, ob ein Datenelement mit dem eingegebenen Schlüssel (in Form einer
    Zeichenkette) vorhanden ist.
    - parameters:
        - suchSchlüssel: Schlüssel, nach dem gesucht wird als Zeichenkette
    - returns: true, im Erfolgsfall, false sonst.
    */
    override func IstVorhanden(suchSchlüssel: String) -> Bool
    {
        return false
    }

    /**
    Berechnet die Höhe des Baums und gibt diesen Wert zurück
    - parameters:
        - aktTiefe: Tiefe des aktuellen Knotens
    - returns: die Höhe = maximale Tiefe (der bisher besuchten Knoten)
    */
    override func HöheGeben(aktTiefe: Int) -> Int
    {
        return aktTiefe - 1
    }

    /**
    Gibt die Tiefe des Knotens an, in dem ein Datenelement mit dem Schlüssel suchSchluessel gespeichert ist.
    - parameters:
        - suchSchlüssel: Schlüssel, nach dem gesucht wird als Zeichenkette
        - tiefe: die Tiefe des aktuellen Knotens
    - returns: Tiefe als positive Zahl, bzw. -1 falls der suchSchlüssel nicht vorhanden ist.
    */
    override func TiefeGeben(suchSchlüssel: String, tiefe: Int) -> Int
    {
        return -1
    }
}

/**
Abstrakte Klasse Baumelement zur Realisierung einer Baumstruktur mit Hilfe des Entwurfsmusters
Kompositum: Die Klasse legt ausschließlich die Schnittstelle für die Klassen
Knoten und Abschluss in Form von Methoden fest.

- author: Peter Brichzin
- version: 23.5.24
*/
class KnotenBaum: Baumelement
{
    /** Datenelement des Knotens */
    private var daten: Wörterbucheintrag

    /** Referenz auf den linken Nachfolger */
    private var linkerNachfolger: Baumelement
    /** Referenz auf den rechten Nachfolger */
    private var rechterNachfolger: Baumelement

    /**
    Konstruktor für Objekte der Klasse Knoten ohne linken und rechten Nachfolger, d.h. es werden
    zwei Abschluss-Objekte erzeugt und den entsprechenden Referenzattributen zugewiesen.
    - parameters:
        - datenNeu: Referenz auf ein Objekt der KLasse Woerterbucheintrag, das vom Knoten verwaltet wird.
    */
    init(datenNeu: Wörterbucheintrag)
    {
        daten = datenNeu
        linkerNachfolger = AbschlussBaum()
        rechterNachfolger = AbschlussBaum()
        super.init()
    }

    /**
    Konstruktor für Objekte der Klasse Knoten mit Nachfolgern
    - parameters:
        - datenNeu: Referenz auf ein Objekt der KLasse Woerterbucheintrag, das vom Knoten verwaltet wird.
        - linkerNachfolgerNeu: der linke Nachfolger des Knotens
        - rechterNachfolgerNeu: der rechte Nachfolger des Knotens
    */
    init(datenNeu: Wörterbucheintrag, linkerNachfolgerNeu: Baumelement, rechterNachfolgerNeu: Baumelement)
    {
        daten = datenNeu
        linkerNachfolger = linkerNachfolgerNeu
        rechterNachfolger = rechterNachfolgerNeu
        super.init()
    }

    /**
    Fügt ein Datenelement, falls sein Schlüssel noch nicht vorhanden ist,
    in die bestehende Datenstruktur ein. D.h. die Eigenschaft geordneter Binärbaum
    bleibt erhalten.
    - parameters:
        - datenNeu: das Datenelement des einzufügenden Knoten
    - returns: (neues) Nachfolgeelement des Aufrufers, null falls es kein Datenelement mit dem eingegebenen Schlüssel gibt
    */
    override func Einfügen(datenNeu: Wörterbucheintrag) -> Baumelement
    {
        // Falls es den Wert schon gibt, Einfügen abbrechen.
        if daten.IstGleich(dvergleich: datenNeu)
        {
            print("Die Daten existieren schon!");
        }
        // Abfrage ob im rechten oder linken Teilbaum eingefügt werden muss
        else
        {
            if daten.IstGrößerAls(dvergleich: datenNeu)
            {
                linkerNachfolger = linkerNachfolger.Einfügen(datenNeu: datenNeu)
            }
            else
            {
                rechterNachfolger = rechterNachfolger.Einfügen(datenNeu: datenNeu)
            }
        }
        return self
    }

    /**
    Sucht ein Datenelement, das über seinen Schlüssel identifiziert wird
    und gibt dieses aus.
    - parameters:
        - suchSchlüssel: Schlüssel nach dem gesucht wird als Zeichenkette
    - returns: gesuchtes Datenelement bzw. null, falls die Suche erfolglos ist
    */
    override func Suchen(suchSchlüssel: String) -> Wörterbucheintrag?
    {
        if daten.SchlüsselIstGleich(vergleichsSchlüssel: suchSchlüssel)
        {
            return daten
        }
        else
        {
            if daten.SchlüsselIstGrößerAls(vergleichsSchlüssel: suchSchlüssel)
            {
                return linkerNachfolger.Suchen(suchSchlüssel: suchSchlüssel) // Rückgabewert wird nach unten abgefragt und dann nach oben durchgereicht
            }
            else
            {
                return rechterNachfolger.Suchen(suchSchlüssel: suchSchlüssel) // Rückgabewert wird nach unten abgefragt und dann nach oben durchgereicht
            }
        }
    }

    /**
    Überprüft, ob ein Datenelement mit dem eingegebenen Schlüssel (in Form einer
    Zeichenkette) vorhanden ist.
    - parameters:
        - suchSchlüssel: Schlüssel, nach dem gesucht wird als Zeichenkette
    - returns: true, im Erfolgsfall, false sonst.
    */
    override func IstVorhanden(suchSchlüssel: String) -> Bool
    {
        if daten.SchlüsselIstGleich(vergleichsSchlüssel: suchSchlüssel)
        {
            return true
        }
        else
        {
            if daten.SchlüsselIstGrößerAls(vergleichsSchlüssel: suchSchlüssel)
            {
                return linkerNachfolger.IstVorhanden(suchSchlüssel: suchSchlüssel) // Rückgabewert wird nach unten abgefragt und dann nach oben durchgereicht
            }
            else
            {
                return rechterNachfolger.IstVorhanden(suchSchlüssel: suchSchlüssel) // Rückgabewert wird nach unten abgefragt und dann nach oben durchgereicht
            }
        }
    }

    /**
    Berechnet die Höhe des Baums und gibt diesen Wert zurück
    - parameters:
        - aktTiefe: Tiefe des aktuellen Knotens
    - returns: die Höhe = maximale Tiefe (der bisher besuchten Knoten)
    */
    override func HöheGeben(aktTiefe: Int) -> Int
    {
        let maxHöheLinkererTeilbaum = linkerNachfolger.HöheGeben(aktTiefe: aktTiefe + 1)
        let maxHöheRechterTeilbaum = rechterNachfolger.HöheGeben(aktTiefe: aktTiefe + 1)
        if maxHöheLinkererTeilbaum > maxHöheRechterTeilbaum
        {
            return maxHöheLinkererTeilbaum
        }
        else
        {
            return maxHöheRechterTeilbaum
        }
    }

    /**
    Gibt die Tiefe des Knotens an, in dem ein Datenelement mit dem Schlüssel suchSchluessel gespeichert ist.
    - parameters:
        - suchSchlüssel: Schlüssel, nach dem gesucht wird als Zeichenkette
        - tiefe: die Tiefe des aktuellen Knotens
    - returns: Tiefe als positive Zahl, bzw. -1 falls der suchSchlüssel nicht vorhanden ist.
    */
    override func TiefeGeben(suchSchlüssel: String, tiefe: Int) -> Int
    {
        if daten.SchlüsselIstGleich(vergleichsSchlüssel: suchSchlüssel)
        {
            return tiefe
        }
        else
        {
            if daten.SchlüsselIstGrößerAls(vergleichsSchlüssel: suchSchlüssel)
            {
                return linkerNachfolger.TiefeGeben(suchSchlüssel: suchSchlüssel, tiefe: tiefe + 1)
            }
            else
            {
                return rechterNachfolger.TiefeGeben(suchSchlüssel: suchSchlüssel, tiefe: tiefe + 1)
            }
        }
    }
}

/**
Die Klasse Binbaum ist die Grundstruktur eines geordneten Binärbaums.
Konkret wird hier der Binärbaum genutzt um ein einfaches Wörterbuch umzusetzen,
deshalb sind die Daten von der Klasse Woerterbucheintrag.
Die Methoden die ein Objekt dieser Klasse anbietet, leiten den Aufruf an den
Wurzelknoten weiter, wenn der Baum nicht leer ist.
(Implementierung mit Entwurfsmuster Kompositum)

- author: Peter Brichzin
- version: 23.5.24
*/
class Binbaum
{
    /** Die wurzel des Bauns */
    private var wurzel: Baumelement

    /**
    Konstruktor für Objekte der Klasse Binbaum
    Ein leerer Baum wird erzeugt: Er besteht nur aus einem Abschluss-Objekt.
    */
    init()
    {
        wurzel = AbschlussBaum()
    }

    /**
    Fügt ein Datenelement sortiert in den geordneten Binärbaum ein,
    d.h. die Eigenschaft, dass der Baum geordnet ist bleibt bei jedem
    Einfügevorgang erhalten.
    - parameters:
        - datenNeu: neues Datenelement
    */
    func Einfügen(datenNeu: Wörterbucheintrag)
    {
        wurzel = wurzel.Einfügen(datenNeu: datenNeu)
    }

    /**
    Sucht ein Datenelement, das über seinen Schlüssel
    identifiziert wird, und gibt es aus.
    - parameters:
        - suchSchlüssel: Schlüssel, nach dem gesucht wird
    - returns: gesuchtes Datenelement bzw. nil, falls die Suche erfolglos ist
     */
    func Suchen(suchSchlüssel: String) -> Wörterbucheintrag?
    {
        return wurzel.Suchen(suchSchlüssel: suchSchlüssel)
    }

    /**
    Überprüft, ob ein Datenelement mit dem eingegebenen Schlüssel (in Form einer
    Zeichenkette) vorhanden ist.
    - parameters:
        - suchSchlüssel: Schlüssel, nach dem gesucht wird als Zeichenkette
    - returns: true, im Erfolgsfall, false sonst.
    */
    func IstVorhanden(suchSchlüssel: String) -> Bool
    {
        return wurzel.IstVorhanden(suchSchlüssel: suchSchlüssel)
    }

    /**
    Berechnet die Höhe des Baums und gibt diesen Wert zurück
    - returns: Höhe des Baums
    */
    func HöheGeben() -> Int
    {
        return wurzel.HöheGeben(aktTiefe: 0)
    }

    /**
    Gibt die Tiefe des Knotens an, in dem ein Datenelement mit dem Schlüssel suchSchlüssel gespeichert ist.
    parameters:
        - suchSchlüssel: Schlüssel, nach dem gesucht wird als Zeichenkette
    - returns: Tiefe als positive Zahl, bzw. -1 falls der suchSchlüssel nicht vorhanden ist.
    */
    func TiefeGeben(suchSchlüssel: String) -> Int
    {
        return wurzel.TiefeGeben(suchSchlüssel: suchSchlüssel, tiefe: 0)
    }
}
