import CoreGraphics

/**
Zeichnet den Pythagorasbaum.

- author: Albert Wiedemann
- version: 1.0
 */
class PythagorasBaum
{
    /** die Turtle */
    let turtle: Turtle
    /** Seite a des rechtwinkeligen Ausgangsdreiecks */
    let a: Double
    /** Seite b des rechtwinkeligen Ausgangsdreiecks */
    let b: Double
    /** Seite c^2 des rechtwinkeligen Ausgangsdreiecks */
    let c2: Double
    /** Seitenverhältnis für die Berechnung der C-Punkte der angefügten Dreiecke */
    let bc: Double
    /** Seitenverhältnis für die Berechnung der C-Punkte der angefügten Dreiecke */
    let abc: Double
    
    /**
	Besetzt die Konstanten und baut den Baum auf.
	- parameters:
		- tiefe: die Rekursionstiefe
    */
    init(tiefe: Int)
    {
        turtle = Turtle()
        a = 300.0
        b = 400.0
        c2 = a * a + b * b
        bc = (b * b) / c2
        abc = a * b / c2
        ViereckZeichnen(ax: 400.0, ay: 100.0, bx: 500.0, by: 100.0, cx: 500.0, cy: 200.0, dx: 400.0, dy: 200.0)
        SchrittAusführen(ax: 400.0, ay: 200.0, bx: 500.0, by: 200.0, tiefe: tiefe)
    }
    
    /**
	Berechnet die Richtung von Startpunkt zum Zielpunkt.
	- parameters:
		- xStart: x-Koordinate des Startpunkts
		- yStart: y-Koordinate des Startpunkts
		- xZiel: x-Koordinate des Zielpunkts
		- yZiel: y-Koordinate des Zielpunkts
	- returns: Richtungswinkel zum Zielpunkt
    */
    private func RichtungGeben(xStart: Double, yStart: Double, xZiel: Double, yZiel: Double) -> Int
    {
        let dx = xZiel - xStart
        let dy = yZiel - yStart
        var richtung = 0
        if dx == 0
        {
            if dy > 0
            {
                richtung = 270
            }
            else if dy < 0
            {
                richtung = 90
            }
        }
        else if dx > 0
        {
            richtung = -Int(round(atan(dy / dx) * 180.0 / Double.pi))
        }
        else
        {
            richtung = 180 - Int(round(atan(dy / dx) * 180.0 / Double.pi))
        }
        return richtung
    }
    
    /**
	Berechnet die Weglänge von Startpunkt zum Zielpunkt.
	- parameters:
		- xStart: x-Koordinate des Startpunkts
		- yStart: y-Koordinate des Startpunkts
		- xZiel: x-Koordinate des Zielpunkts
		- yZiel: y-Koordinate des Zielpunkts
	- returns: Weglänge zum Zielpunkt
     */
    private func LängeGeben(xStart: Double, yStart: Double, xZiel: Double, yZiel: Double) -> Double
    {
        let dx = xZiel - xStart
        let dy = yZiel - yStart
        return sqrt(dx * dx + dy * dy)
    }

    /**
	Zeichnet das Dreieck mit den angegebenen Eckpunkten
	- parameters:
		- ax: x-Koordinate der Ecke A
		- ay: y-Koordinate der Ecke A
		- bx: x-Koordinate der Ecke B
		- by: y-Koordinate der Ecke B
		- cx: x-Koordinate der Ecke C
		- cy: y-Koordinate der Ecke C
    */
    private func DreieckZeichnen(ax: Double, ay: Double, bx: Double, by: Double, cx: Double, cy: Double)
    {
        turtle.PositionSetzen(x: Int(ax), y: Int(ay))
        turtle.WinkelSetzen(winkel: RichtungGeben(xStart: ax, yStart: ay, xZiel: bx, yZiel: by))
        turtle.Gehen(länge: LängeGeben(xStart: ax, yStart: ay, xZiel: bx, yZiel: by))
        turtle.WinkelSetzen(winkel: RichtungGeben(xStart: bx, yStart: by, xZiel: cx, yZiel: cy))
        turtle.Gehen(länge: LängeGeben(xStart: bx, yStart: by, xZiel: cx, yZiel: cy))
        turtle.WinkelSetzen(winkel: RichtungGeben(xStart: cx, yStart: cy, xZiel: ax, yZiel: ay));
        turtle.Gehen(länge: LängeGeben(xStart: cx, yStart: cy, xZiel: ax, yZiel: ay))
    }

    /**
	Zeichnet das Viereck mit den angegebenen Eckpunkten
	- parameters:
		- ax: x-Koordinate der Ecke A
		- ay: y-Koordinate der Ecke A
		- bx: x-Koordinate der Ecke B
		- by: y-Koordinate der Ecke B
		- cx: x-Koordinate der Ecke C
		- cy: y-Koordinate der Ecke C
		- dx: x-Koordinate der Ecke D
		- dy: y-Koordinate der Ecke D
    */
    private func ViereckZeichnen (ax: Double, ay: Double, bx: Double, by: Double, cx: Double, cy: Double, dx: Double, dy: Double)
    {
        turtle.PositionSetzen(x: Int(ax), y: Int(ay))
        turtle.WinkelSetzen(winkel: RichtungGeben(xStart: ax, yStart: ay, xZiel: bx, yZiel: by))
        turtle.Gehen(länge: LängeGeben(xStart: ax, yStart: ay, xZiel: bx, yZiel: by))
        turtle.WinkelSetzen(winkel: RichtungGeben(xStart: bx, yStart: by, xZiel: cx, yZiel: cy))
        turtle.Gehen(länge: LängeGeben(xStart: bx, yStart: by, xZiel: cx, yZiel: cy))
        turtle.WinkelSetzen(winkel: RichtungGeben(xStart: cx, yStart: cy, xZiel: dx, yZiel: dy))
        turtle.Gehen(länge: LängeGeben(xStart: cx, yStart: cy, xZiel: dx, yZiel: dy))
        turtle.WinkelSetzen(winkel: RichtungGeben(xStart: dx, yStart: dy, xZiel: ax, yZiel: ay))
        turtle.Gehen(länge: LängeGeben(xStart: dx, yStart: dy, xZiel: ax, yZiel: ay))
    }

	/**
	Führt einen Rekursionsschritt aus
	- parameters:
		- ax: x-Koordinate der Ecke A der Basislinie
		- ay: y-Koordinate der Ecke A der Basislinie
		- bx: x-Koordinate der Ecke B der Basislinie
		- by: y-Koordinate der Ecke B der Basislinie
		- tiefe: die Rekursionstiefe
     */
    private func SchrittAusführen(ax: Double, ay: Double, bx: Double, by: Double, tiefe: Int)
    {
        if tiefe > 0
        {
            let cx = ax + (bx - ax) * bc - (by - ay) * abc
            let cy = ay + (by - ay) * bc + (bx - ax) * abc
            DreieckZeichnen(ax: ax, ay: ay, bx: bx, by: by, cx: cx, cy: cy)
            var p1x = ax - (cy - ay)
            var p1y = ay + (cx - ax)
            var p2x = cx - (cy - ay)
            var p2y = cy + (cx - ax)
            ViereckZeichnen(ax: ax, ay: ay, bx: cx, by: cy, cx: p2x, cy: p2y, dx: p1x, dy: p1y)
            SchrittAusführen(ax: p1x, ay: p1y, bx: p2x, by: p2y, tiefe: tiefe - 1)
            p1x = cx - (by - cy)
            p1y = cy + (bx - cx)
            p2x = bx - (by - cy)
            p2y = by + (bx - cx)
            ViereckZeichnen(ax: cx, ay: cy, bx: bx, by: by, cx: p2x, cy: p2y, dx: p1x, dy: p1y)
            SchrittAusführen(ax: p1x, ay: p1y, bx: p2x, by: p2y, tiefe: tiefe - 1)
        }
    }
}

PythagorasBaum(tiefe: 5)
