
/**
Zeichnet die Sierpinski-Kurve

- author: Albert Wiedemann
- version: 1.0
*/
class SierpinskiKurve
{
    /** Die Turtle */
    private let t: Turtle
    /** Linienlänge */
    private var länge: Double
    /** Länge der Kurzen Linien */
    private var länge2: Double

    /**
	Legt die Turtle an und startet die Zeichnung
	- parameters:
		- tiefe: Rekursionstiefe
    */
    init(tiefe: Int)
    {
        t = Turtle()
        länge = 125.0
        var x0 = 400.0
        var y0 = 250.0 - länge
        for _ in 1 ... tiefe
        {
            x0 -= länge
            länge = länge / 2.0
            y0 -= länge
        }
        länge2 = länge * 1.414213562373095
        länge = länge * 2.0
        t.StiftHeben()
        t.PositionSetzen(x: Int(x0), y: Int(y0))
        t.StiftSenken()
        ElementAZeichnen(tiefe: tiefe)
        t.WinkelSetzen(winkel: 315)
        t.Gehen(länge: länge2)
        ElementBZeichnen(tiefe: tiefe)
        t.WinkelSetzen(winkel: 225)
        t.Gehen(länge: länge2)
        ElementCZeichnen(tiefe: tiefe)
        t.WinkelSetzen(winkel: 135)
        t.Gehen(länge: länge2)
        ElementDZeichnen(tiefe: tiefe)
        t.WinkelSetzen(winkel: 45)
        t.Gehen(länge: länge2)
    }
    
    /**
	Zeichnet das Element A der Sierpinski-Kurve
	- parameters:
		- tiefe: Rekursionstiefe
    */
    private func ElementAZeichnen(tiefe: Int)
    {
        if tiefe > 0
        {
            ElementAZeichnen(tiefe: tiefe - 1)
            t.WinkelSetzen(winkel: 315)
            t.Gehen(länge: länge2)
            ElementBZeichnen(tiefe: tiefe - 1)
            t.WinkelSetzen(winkel: 0)
            t.Gehen(länge: länge)
            ElementDZeichnen(tiefe: tiefe - 1)
            t.WinkelSetzen(winkel: 45)
            t.Gehen(länge: länge2)
            ElementAZeichnen(tiefe: tiefe - 1)
        }
    }
    
    /**
	Zeichnet das Element B der Sierpinski-Kurve
	- parameters:
		- tiefe: Rekursionstiefe
    */
    private func ElementBZeichnen(tiefe: Int)
    {
        if tiefe > 0
        {
            ElementBZeichnen(tiefe: tiefe - 1)
            t.WinkelSetzen(winkel: 225)
            t.Gehen(länge: länge2)
            ElementCZeichnen(tiefe: tiefe - 1)
            t.WinkelSetzen(winkel: 270)
            t.Gehen(länge: länge)
            // Rest von ElementBZeichnen ergänzen
        }
    }
    
    /**
	Zeichnet das Element C der Sierpinski-Kurve
	- parameters:
		- tiefe: Rekursionstiefe
    */
    private func ElementCZeichnen (tiefe: Int)
    {
        if tiefe > 0
        {
            ElementCZeichnen(tiefe: tiefe - 1)
            t.WinkelSetzen(winkel: 135)
            t.Gehen(länge: länge2)
            // Rest von ElementCZeichnen ergänzen
        }
    }
    
    /**
	Zeichnet das Element D der Sierpinski-Kurve
	- parameters:
		- tiefe: Rekursionstiefe
    */
    private func ElementDZeichnen (tiefe: Int)
    {
        if tiefe > 0
        {
            // ElementDZeichnen ergänzen
        }
    }
}
